#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>
#include <sched.h>
#include "perf_util.h"

/* Create counters for process to use and then run the process, passing the fds in environment variables */
int main(int argc, char* argv[]) {
  perf_event_desc_t * fds = NULL;
  int num_fds = 0;
  int i = 0;
  int j,ret;
  char * fd_string;
  char * cursor;
  char * events;
  char * event_list;
  char *src, *dest;
  int percore = 0;
  int nr_cpu = 1;
  

  if (argc <= 2)
    errx(1, "usage: [-aNR_CPU] launcher EVENT1,EVENT2,...,EVENTN /path/to/java ...");


  if (strncmp(argv[1],"-a",2)){
    events = argv[1];
  }  
  else {
    percore = 1;
    events = argv[2];
    nr_cpu = atoi(argv[1] + 2);    
    printf("Set counters for %d cpu\n",nr_cpu);
  }
    

  ret = pfm_initialize();
  if (ret != PFM_SUCCESS)
    errx(1, "Cannot initialize library: %s", pfm_strerror(ret));

  ret = perf_setup_list_events(events, &fds, &num_fds);
  if (ret || !num_fds)
    errx(1, "cannot setup events for %s",events);
  
  fd_string = (char*)malloc(num_fds * 5 * nr_cpu);
  if (!fd_string)
    errx(1, "Could not allocate fd string");

  fds[0].fd = -1;
  cursor = fd_string;

  for(j=0; j < nr_cpu; j++) {

    for(i=0; i < num_fds; i++) {  
      fds[i].hw.read_format = PERF_FORMAT_SCALE;
      
      if(percore)
	fds[i].hw.disabled = 0;
      else
	fds[i].hw.disabled = (i == 0);

      if(percore)
	fds[i].hw.inherit = 0;
      else
	fds[i].hw.inherit = 1;

      fds[i].hw.enable_on_exec = 1;

      if(percore){
	fds[i].fd = perf_event_open(&fds[i].hw, -1, j, -1, 0);
      }
      else
	fds[i].fd = perf_event_open(&fds[i].hw, 0, -1, fds[0].fd, 0);

      if (fds[i].fd == -1)
	err(1, "cannot open event %d for %d", i,j);
      
      cursor += sprintf(cursor, "%d ", fds[i].fd);
    }
  }

  *cursor = '\0';


  if(percore){
    event_list = malloc((strlen(events) + i * 10) * nr_cpu);
    if(event_list == NULL)
      errx(1, "can not alloc memory for event_list\n");
    dest = event_list;
    for(i=0; i< nr_cpu; i++){
      src = events;
      while(*src) {
	*dest++ = *src;
	if(*src++ == ','){
	  dest--;
	  dest += sprintf(dest,"_CPU%d ",i);	
	}
      }    
      dest += sprintf(dest,"_CPU%d ",i);
    }
    
    *--dest = 0;
  }
  else{
    cursor = events;
    while(*cursor) {
      if (*cursor == ',') *cursor = ' ';
      cursor++;
    }
    event_list = events;
  }

  setenv("PERF_EVENT_NAMES", event_list, 1);
  setenv("PERF_EVENT_FDS", fd_string, 1);
  
  printf("Event_list is %s\n"
	 "FD_list is %s\n",
	 event_list, fd_string);
  
  if(percore)
    execvp(argv[3], argv + 3);
  else
    execvp(argv[2], argv + 2);
  return 0;
}
