#!/usr/bin/ruby

unless ARGV.length == 2 
	puts "Remember: ./script_generator.rb <inputfile> <outputFile>"
	exit
end

input = ARGV[0]
output = ARGV[1]

colors = ["#8B008B","#00FA9A", "#0000FF","#7FFF00","#FF7F50","#008B8B","#B8860B","#A9A9A9","#8FBC8F","#FF1493","#00BFFF","#F0E68C","#FFB6C1","#778899","#FFE4B5","#FFA500","#87CEEB","#FF6347"]

threadColors = [ ["attachedThread","#00FFFF"],
		 ["pgcThread","#A52A2A"],
		 ["vmThread","#006400"],
		 ["watcherThread", "#FFD700"],
		 ["compilerThread","#191970"]]


outputFile = File.open(output + ".gnuplot", "w")

outputFile.puts("set terminal postscript eps enhanced color font \"Arial,12\"")
outputFile.puts("set output \"" + output + ".eps\"")

inputFile = File.open(input, 'r')

title = inputFile.gets.delete("\n")

outputFile.puts("set title \"" + title + "\"")
outputFile.puts("set grid")
outputFile.puts("set xlabel \"Parallelism\"")
outputFile.puts("set ylabel \"Runtime (in seconds)\"")
outputFile.puts("set key outside right center")
outputFile.puts("set style fill solid 1.0 noborder")

maxValX = inputFile.gets.delete("\n")

outputFile.puts("set xrange [-" + maxValX + ":" + maxValX + "]")

outputFile.write("set xtics (")

for i in ((maxValX.to_i) * -1)..(maxValX.to_i)
	outputFile.write( "\"" + i.abs.to_s + "\" " + i.to_s  )
	
	if i < maxValX.to_i
		outputFile.write(",")
	else
		outputFile.write(")\n")
	end
end


class ThreadRecord
	attr_accessor :name, :criticality, :runtime, :parallelism
end

all_threads = Array.new()

while line = inputFile.gets
	if line.chomp.size > 0
		temp = line.split(",")
	
		thread = ThreadRecord.new
		thread.name = temp[0]
		thread.criticality = temp[1].to_i
		thread.runtime = temp[2].to_i
		thread.parallelism = thread.runtime.to_f / thread.criticality.to_f
	
		all_threads.push(thread)
	end
end

#debug tool
#all_threads.each do |thread|
#	puts "name: " + thread.name
#	puts "criticality: " + thread.criticality.to_s
#	puts "runtime: " + thread.runtime.to_s
#	puts "parallelism: " + thread.parallelism.to_s
#end

# calculate max y-value

max_y_range = { "avrora" => 12,
	        "jython" => 12,
	        "luindex" => 3,
	        "lusearch" => 7,
	        "pmd" => 4,
	        "sunflow" => 7,
	        "xalan" => 6,
	        "antlr" => 4,
	        "bloat" => 7,
	        "eclipse" => 40,
	        "fop" => 3,
		"PJBB2005" => 3}

maxY = 0
all_threads.each do |thread|
	maxY += thread.criticality
end
maxY = (maxY.to_f / 1000000000).ceil

benchmark = title.split(" ")[0]

if maxY > max_y_range[benchmark]
	outputFile.puts("set yrange [0:" + maxY.to_s + "]")
else
	outputFile.puts("set yrange [0:" + max_y_range[benchmark].to_s + "]")
end

# code for drawing boxes
base = 0
counter = 1
threadOrder = Array.new
color = ""

while all_threads.length > 0 do
	temp_max = 0
	temp_pos = 0
		
	for i in 0..(all_threads.length-1)
		if all_threads[i].parallelism > temp_max
			temp_max = all_threads[i].parallelism
			temp_pos = i
		end
	end
	
	threadColors.each do |threadColor|
		if all_threads[temp_pos].name.include? threadColor[0] 
			color = threadColor[1]
		end
	end
	
	if color == ""
		color = colors[(counter-1)%colors.length]
	end

	outputFile.puts("set object " + counter.to_s + " rect from " + (temp_max*-1).to_s + "," + base.to_s + " to " + temp_max.to_s + "," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")

	base = base + (all_threads[temp_pos].criticality).to_f/1000000000
	counter = counter + 1
	
	threadOrder.push all_threads[temp_pos].name
	
	all_threads.delete_at(temp_pos)
	color = ""
end

outputFile.write("plot ")

color = ""
for i in 0..(threadOrder.length-1)

	threadColors.each do |threadColor|
		if threadOrder[threadOrder.length - 1 - i].include? threadColor[0] 
			color = threadColor[1]
		end
	end
	
	if color == ""
		color = colors[(threadOrder.length - 1 -i)%colors.length]
	end

	outputFile.write( "-1 with boxes title \"" + threadOrder[threadOrder.length - 1 - i] + "\" lc rgb \"" + color + "\"")
	if i < threadOrder.length-1
		outputFile.write(", ")
	end
	
	color = ""
end

outputFile.close
inputFile.close
