#!/usr/bin/ruby

require 'fileutils.rb'

unless ARGV.length == 3
	puts "Remember: ./runPJBB.rb <times_min_heap> <num_GC_threads> <num_app_threads>"
	exit
end

min_heap = 200

times_min_heap = ARGV[0].to_f	   
num_GC_threads = ARGV[1].to_i
num_app_threads = ARGV[2].to_i

results_dir = "/scratch/kdubois/results/"
scripts_dir = "/scratch/kdubois/critical_threads/scripts/"
 
dirName = times_min_heap.round().to_s + "x_minHeap_" + num_GC_threads.to_s + "_GCs_" + num_app_threads.to_s + "_app_openjdk"
output_dir = results_dir + dirName 

FileUtils.mkdir_p output_dir

puts `echo java > /proc/profiled_process`

# run the benchmark + parse thread names
heap_size = (min_heap * times_min_heap).round()
	
title = "PJBB2005_heap" + heap_size.to_s + "MB_GC-threads" + num_GC_threads.to_s + "_app-threads" + num_app_threads.to_s
	
puts `echo clear_stack > /proc/signal_action ; echo init_stack > /proc/signal_action`
puts `echo #{title} > /proc/program_name_output`
		
puts `taskset -c 0-7,16-23 /scratch/kdubois/openjdk/jdk6/build/linux-amd64/bin/java -Xms#{heap_size}M -Xmx#{heap_size}M  -server -Xss128K -XX:+UseParallelGC -XX:+UseParallelOldGC -XX:+UseBiasedLocking -XX:+AggressiveOpts -XX:ParallelGCThreads=#{num_GC_threads} -Dprobes=StopJIT -Dprobe.stopjit.iteration=13 -cp /scratch/kdubois/jikes/bin/probes/probes.jar:/scratch/kdubois/jikes/benchmarks/pjbb2005/jbb.jar:/scratch/kdubois/jikes/benchmarks/pjbb2005/check.jar spec.jbb.JBBmain -propfile /scratch/kdubois/jikes/benchmarks/pjbb2005/SPECjbb-#{num_app_threads}x10000.props -c probe.PJBB2005Callback -n 16 > #{output_dir}/PJBB2005.log 2>&1`
	
puts `#{scripts_dir}bm_output_parser.rb #{output_dir}/PJBB2005.log #{output_dir}/PJBB2005.threadNames`
  
# parse results
puts `#{scripts_dir}results_log_parser.rb #{output_dir}`

# format data for gnuplot-script generator 
Dir.chdir(output_dir) do
	Dir.glob("*.data").each do |file|
		puts `#{scripts_dir}input_plot_generator.rb #{file} #{output_dir}`
	end
end

# invoke gnuplot-script generator
Dir.chdir(output_dir) do
	Dir.glob("*.data.formatted").each do |file|
		temp = file.gsub(".data.formatted","")
		puts `#{scripts_dir}gnuplot_script_generator_openjdk.rb #{output_dir}/#{file} #{output_dir}/#{temp}`
	end
end

# invoke gnuplot
Dir.chdir(output_dir) do
	Dir.glob("*.gnuplot").each do |file|
		puts `gnuplot #{output_dir}/#{file}`
	end
end
   	   
